#!/usr/bin/perl
#
# Copyright 2009, Fonality Inc. ALL RIGHTS RESERVED.

use BerkeleyDB;
use IO::Socket;
use Data::Dumper;
use strict;

# global sock
my $sock;
my $db = "/var/lib/asterisk/heatmap.bdb";

# determine our ami password
my $secret;
my $username = 'local_mgr';
open(MGR, "</etc/asterisk/manager.conf") || die "Can't read /etc/asterisk/manager.conf: $!";
while (<MGR>)
{
	if (/^\[$username\]/)
	{
		while (<MGR>)
		{
			if (/^secret\s*=\s*(\S+)/)
			{
				$secret = $1;
				last;
			}
		}
		last;
	}
}
close(MGR);
if (!$secret)
{
	die "Can't find AMI password for local_mgr in /etc/asterisk/manager.conf!";
}

my $time;

my $DEBUG = shift || 0;

# make connection for asterisk remote manager
mgr_connect();

# open our db
my %h;
tie(%h, "BerkeleyDB::Hash",
	-Filename => $db,
	-Flags    => DB_CREATE)
	or die "Cannot open file $db: $! $BerkeleyDB::Error\n" ;
untie %h;

if (!$DEBUG)
{
	my $pid = fork();
	if ($pid)
	{
		print "Running in BG ($pid)\n";
		exit(0);
	}

	close(STDIN);
	close(STDERR);
	close(STDOUT);
}

my %channels;
my %data;
my %dtmf;
while (1)
{
	%data = get_data();
	if ($DEBUG)
	{
		print Dumper(\%data) if$data{userevent} ne "DBPut"&&$data{event} ne"PeerStatus";
	}

	# set dtmf
	if ($data{event} eq 'DTMF')
	{
		$dtmf{$data{channel}} = $data{digit};
	}

	# check if dtmf was valid
	elsif ($data{event} eq 'Newexten' && $data{extension} eq $dtmf{$data{channel}})
	{
		print "$channels{$data{channel}} -> $dtmf{$data{channel}}\n" if $DEBUG;
		tie(%h, "BerkeleyDB::Hash",
			-Filename => $db,
			-Flags    => DB_CREATE)
			or die "Cannot open file $db: $! $BerkeleyDB::Error\n" ;
		$h{$channels{$data{channel}} . "-" . $dtmf{$data{channel}}}++;
		untie %h;
	}

	# remember what context this channel is in
	if ($data{event} eq 'Newexten' && $data{application} eq 'Goto' && $data{appdata} =~ /^(.*),.*,.*$/)
	{
		$channels{$data{channel}} = $1;
		tie(%h, "BerkeleyDB::Hash",
			-Filename => $db,
			-Flags    => DB_CREATE)
			or die "Cannot open file $db: $! $BerkeleyDB::Error\n" ;
		$h{"conf:$channels{$data{channel}}"}++;
		untie %h;
	}
}
exit;


# remore manager simple send function
sub ssend
{
	my $data = shift;
	my $noconnect = shift;
	my $odata = $data;

	$data .= "\n";
	$data =~ s/\n/\r\n/gs;

	eval
	{
		$sock->send($data);
	};
	if ($@ && !$noconnect)
	{
		mgr_connect();
		ssend($odata, 1);
	}
}

# Remote manager send function that waits for a response.
sub srequest
{
	my $data = shift;
	my $noconnect = shift;
	my $odata = $data;

	$data .= "\n";
	$data =~ s/\n/\r\n/gs;

	eval
	{
		$sock->send ($data);
	};
	if ($@ && !$noconnect)
	{
		mgr_connect();
		return srequest($odata, 1);
	}

	# Now get a response.
	my $response;
	my $timeout = 20;
	eval
	{
		local $SIG{ALRM} = sub {
			die "Timed out!";
		};
		alarm $timeout;
		while (<$sock>)
		{
			$response .= $_;
			if ($_ =~ /^\-\-END COMMAND\-\-/)
			{
				last;
			}
		}
	};

	return $response;
}


# gets data from an asterisk manager response
# and returns it in a readable form
sub get_data
{
        my %data;
        while (<$sock>)
        {
                my @pair = split(/:\s*/, $_, 2);

                $pair[0] = lc($pair[0]);
                # if there's data
                if (@pair == 2)
                {
                        $pair[1] =~ s/\r?\n$//;
                        $data{$pair[0]} = $pair[1];
                }
                # end of data from server
                else
                {
                        last;
                }
        }

        return %data;
}

sub mgr_connect
{
	$sock = IO::Socket::INET->new("localhost:5038");
	unless ($sock)
	{
		print STDERR "Can't connect to localhost:5038\n";#...waiting 10 secs\n";
		return;
#		sleep 10;
#		mgr_connect();
	}
	else
	{
		<$sock>;
		ssend("Action: login\nUsername: $username\nSecret: $secret\n");
	}
	return $sock;
}


__DATA__

Event: DTMF
Privilege: dtmf,all
Channel: DAHDI/1-1
Uniqueid: 1255073885.119858
Digit: 2
Direction: Received
Begin: No
End: Yes


Event: Newexten
Privilege: dialplan,all
Channel: DAHDI/1-1
Context: main
Extension: 2
Priority: 1
Application: Goto
AppData: New_support_main,s,1
Uniqueid: 1255073885.119858

